<?php
/**
 * @author Aart-Jan Boor <aart-jan@wemag.nl>
 * @copyright Aart-Jan Boor <aart-jan@wemag.nl>
 *
 * This file is part of Wemag Online Document Viewer.
 *
 * Wemag Online Document Viewer is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Wemag Online Document Viewer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Wemag Online Document Viewer.  If not, see <http://www.gnu.org/licenses/>.
 */

require_once(KT_LIB_DIR . '/plugins/plugin.inc.php');
require_once(KT_LIB_DIR . '/actions/documentaction.inc.php');

define("WEMAG_ODV_LOG_FILE","/var/log/wemag-online-document-viewer.txt");

/**
 * Document action that allows the actual viewing of a document
 *
 */
class WemagOnlineDocumentViewerDocumentAction extends KTDocumentAction{
	// The namespace for this action
	var $sName = 'wemag.OnlineDocumentViewerPlugin.actions.document';

	// The permission required to see this action on the menu
	var $sPermissionName = 'ktcore.permissions.read';

	var $aAcceptedMimeTypes = array('doc', 'ods', 'odt', 'ott', 'txt', 'rtf', 'sxw', 'stw',
            'xml' , 'pdb', 'psw', 'ods', 'ots', 'sxc',
            'stc', 'dif', 'dbf', 'xls', 'xlt', 'slk', 'csv', 'pxl',
            'odp', 'otp', 'sxi', 'sti', 'ppt', 'pot', 'sxd', 'odg',
            'otg', 'std', 'asc','pdf');
	
	var $aImageMimeTypes = array('png','jpg','gif','bmp','jpeg','jpe');

	/**
	 * Method for getting the MIME type extension for the current document.
	 * Source: KnowledgeTree Community Edition v3.5.2c
	 *
	 * @return string mime time extension
	 */
	private function getMimeExtension() {

		if($this->oDocument == null || $this->oDocument == "" || PEAR::isError($this->oDocument) ) return _kt('Unknown Type');

		$oDocument = $this->oDocument;
		$iMimeTypeId = $oDocument->getMimeTypeID();
		$mimetypename = KTMime::getMimeTypeName($iMimeTypeId); // mime type name

		$sTable = KTUtil::getTableName('mimetypes');
		$sQuery = "SELECT filetypes FROM " . $sTable . " WHERE mimetypes = ?";
		$aQuery = array($sQuery, array($mimetypename));
		$res = DBUtil::getResultArray($aQuery);
		if (PEAR::isError($res)) {
			return $res;
		} else if (count($res) != 0){
			return $res[0]['filetypes'];
		}

		return _kt('Unknown Type');
	}

	function getName(){
		return _kt("View Online");
	}

	/**
	 * Returns display name for use in document sidebar.
	 *
	 * @return display name
	 */
	function getDisplayName() {
		$cmdpath = KTUtil::findCommand('externalBinary/python');
		// Check if openoffice and python are available
		if($cmdpath != false && file_exists($cmdpath) && !empty($cmdpath)) {
			$sDocType = $this->getMimeExtension();
			// make sure that the selected document is of an acceptable extension
			foreach(array_merge($this->aAcceptedMimeTypes,$this->aImageMimeTypes) as $acceptType){
				if($acceptType == $sDocType){
					return _kt('View Online')." <img src=\"plugins/WemagOnlineDocumentViewer/img/edit-find.png\" />";
				}
			}
		}
		return '';
	}

	private function displayImage(){
		return "<img src=\"plugins/WemagOnlineDocumentViewer/getDoc.php?doc=".$this->oDocument->getId()."\" />";
	}

	/**
	 * Converts document into an acceptable format for the viewer, and loads the document into the viewer.
	 *
	 * @return unknown
	 */
	function do_main() {
		$props = array();

		//check if we're dealing with an image
		$sDocType = $this->getMimeExtension();
		foreach($this->aImageMimeTypes as $acceptType){
			if($acceptType == $sDocType){
				return $this->displayImage();
			}
		}

		//check if file type is supported
		$accept = false;
		foreach($this->aAcceptedMimeTypes as $acceptType){
			if($acceptType == $sDocType){
				$accept = true;
			}
		}
		if($accept == false){
			$this->addErrorMessage(_kt('An error occurred viewing the document - document type not supported'));
			redirect(generateControllerLink('viewDocument',sprintf('fDocumentId=%d',$oDocument->getId())));
			exit(0);
		}

		//Retrieve properties of converted document.
		if(file_exists(KT_DIR."/var/wemagonlinedocuments/".$this->oDocument->getId().".swf")){
			$props = unserialize(file_get_contents(KT_DIR."/var/wemagonlinedocuments/".$this->oDocument->getId().".swf.prop"));
		}

		//check if the document already has been converted and if the converted document is still the most recent version.
		if(!file_exists(KT_DIR."/var/wemagonlinedocuments/".$this->oDocument->getId().".swf") || (isset($props['content_version'])
		&& $props['content_version'] != $this->oDocument->getContentVersionId())){
			require_once("PdfToSwfConverter.php");
			$props['content_version'] = $this->oDocument->getContentVersionId();

			$oDocument = $this->oDocument;
			$oStorage =& KTStorageManagerUtil::getSingleton();
			$oConfig =& KTConfig::getSingleton();
			$cmdpath = KTUtil::findCommand('externalBinary/python');

			//path to document on FS
			$sPath = sprintf("%s/%s", $oConfig->get('urls/documentRoot'), $oStorage->getPath($oDocument));

			//We don't have to convert pdfs to pdf since.... they're a pdf already ;-)
			if($this->getMimeExtension() != 'pdf'){
				// Check if openoffice and python are available
				if($cmdpath == false || !file_exists($cmdpath) || empty($cmdpath)) {
					// Set the error messsage and redirect to view document
					$this->addErrorMessage(_kt('An error occurred converting the document to PDF - please contact the system administrator. Python binary not found.'));
					redirect(generateControllerLink('viewDocument',sprintf('fDocumentId=%d',$oDocument->getId())));
					exit(0);
				}

				//does the document exist on the server?
				if (file_exists($sPath)) {

					//convert document to PDF, code taken from KT CE 3.5.2c

					// Get a tmp file
					$sTempFilename = tempnam('/tmp', 'ktpdf');

					// We need to handle Windows differently
					if (substr( PHP_OS, 0, 3) == 'WIN') {
						$cmd = "\"" . $cmdpath . "\" \"". KT_DIR . "/bin/openoffice/pdfgen.py\" \"" . $sPath . "\" \"" . $sTempFilename . "\"";
						$cmd = str_replace( '/','\\',$cmd);
						$cmd = str_replace( '\\\\','\\',$cmd); // FIX by isorokin
						$res = `"$cmd" 2>&1`;
					} else {
						$cmd = $cmdpath . ' ' . KT_DIR . '/bin/openoffice/pdfgen.py ' . escapeshellcmd($sPath) . ' ' . escapeshellcmd($sTempFilename);
						$res = shell_exec($cmd." 2>&1");
					}

					//now lets turn the PDF into a swf movieclip
					if (file_exists($sTempFilename)) {
						PdfToSwfConverter::convertPdfToSwf($sTempFilename ,KT_DIR."/var/wemagonlinedocuments/".$this->oDocument->getId().".swf",$props);
						unlink($sTempFilename);
					}else{
						$this->addErrorMessage(_kt('An error occurred converting the document to PDF - please contact the system administrator. Python binary not found.'));
						redirect(generateControllerLink('viewDocument',sprintf('fDocumentId=%d',$oDocument->getId())));
						exit(0);
					}
				}else{
					$this->addErrorMessage(_kt('An error occurred viewing this file - file not found'));
					redirect(generateControllerLink('viewDocument',sprintf('fDocumentId=%d',$oDocument->getId())));
					exit(0);
				}
			}else{
				//Document is PDF already, convert it to SWF.
				PdfToSwfConverter::convertPdfToSwf($sPath ,KT_DIR."/var/wemagonlinedocuments/".$this->oDocument->getId().".swf",$props);
			}
		}

		//Get the properties of the current document.
		$props = unserialize(file_get_contents(KT_DIR."/var/wemagonlinedocuments/".$this->oDocument->getId().".swf.prop"));

		//show the viewer and load the document.
		return '<div style="width: '.$props['width'].'px; height: '.$props['height'].'px;"><object classid="clsid:D27CDB6E-AE6D-11cf-96B8-444553540000"
        width="100%"
        height="100%"
        codebase="http://active.macromedia.com/flash5/cabs/swflash.cab#version=8,0,0,0">
        <param name="MOVIE" value="'."plugins/WemagOnlineDocumentViewer/getDoc.php?swf=".$this->oDocument->getId().'">
        <param name="PLAY" value="true">
        <param name="LOOP" value="true">
        <param name="QUALITY" value="high">
        <param name="FLASHVARS" value="zoomtype=3">
          <embed src="'."plugins/WemagOnlineDocumentViewer/getDoc.php?swf=".$this->oDocument->getId().'" width="100%" height="100%"
                 play="true" ALIGN="" loop="true" quality="high"
                 type="application/x-shockwave-flash"
                 flashvars="zoomtype=3"
                 pluginspage="http://www.macromedia.com/go/getflashplayer">
          </embed>
</object></div>';
	}

}

/**
 * This plugin allows online viewing of all kinds of documents.
 */
class WemagOnlineDocumentViewer extends KTPlugin {
	//namespamce
	var $sNamespace = "wemag.WemagOnlineDocumentViewer.plugin";

	/**
	 * Construct plugin
	 *
	 * @param string $sFilename
	 * @return WemagOnlineDocumentViewer plugin object
	 */
	function WemagOnlineDocumentViewer($sFilename = null) {
		$res = parent::KTPlugin($sFilename);
		$this->sFriendlyName = _kt('Wemag Online Document Viewer - View documents online');
		return $res;
	}

	/**
	 * Setup template path, actions, admin pages etc.
	 *
	 */
	function setup() {

		//set up swf dir
		if(!is_dir(KT_DIR."/var/wemagonlinedocuments")){
			die("Please create the directory ".KT_DIR."/var/wemagonlinedocuments"." first in the way described in the installation instructions of this plugin.");
		}

		//create log file
		if(!is_file(KT_DIR.WEMAG_ODV_LOG_FILE)){
			if($fh = @fopen(KT_DIR.WEMAG_ODV_LOG_FILE,"a+b")){
				fclose($fh);
			}
			if(!is_file(KT_DIR.WEMAG_ODV_LOG_FILE)){
				die("Can't create log file: " . KT_DIR.WEMAG_ODV_LOG_FILE.". Please verify if the containing directory exists and permissions are set correctly");
			}
		}

		//Document action
		$this->registerAction('documentaction', 'WemagOnlineDocumentViewerDocumentAction', 'wemag.OnlineDocumentViewerPlugin.actions.document');
	}
}

$oRegistry =& KTPluginRegistry::getSingleton();
$oRegistry->registerPlugin('WemagOnlineDocumentViewer', 'wemag.WemagOnlineDocumentViewer.plugin', __FILE__);

?>
