<?php

/**
 *
 * message_xml.php,   The BitRock Update Server PHP API
 *
 * Classes to obtain the messages, updates and short news from xml format
 *
 * @author BitRock www.BitRock.com
 *
 */



/**
 * Class that parses xml info and obtains the list of properties of each message
 */
class ReadXMLMessages {
    var $parser;    // Auxiliar variable to parse the information
    var $xml;       // Stores a string with the xml information
    var $info;      // Stores all the information in element objects
    var $current;   // Stores the last element

    /**
     *  Class Constructor, receives a string with xml info
     */
    function ReadXMLMessages($xml){
        $this->xml      = $xml;
        $this->current  = null;
        $this->info     = null;
    }

    /**
     * Parse the current info storing it as a tree of TagElements
     */
    function getInfo(){
        $this->parser = xml_parser_create();        

        xml_set_object($this->parser, $this);
        xml_set_element_handler($this->parser, 'startTag', 'endTag');
        xml_set_character_data_handler($this->parser, 'tagData');
        
        $result = xml_parse($this->parser, $this->xml);
        xml_parser_free($this->parser);
        if (!$this->info)
            $result = false;
        return $result;
    }  

    /**
     * Action performed when a new tag starts while parsing the xml
     */  
    function startTag($parser, $name, $attrs = array()){
        $name = str_replace('-','_',strtolower($name));

        if ($this->info == null){
            $this->info = new TagElement($name, null, $attrs);
            $this->current = $this->info;
        }else if ($this->current != null){                        
            $this->current = $this->current->addChildElement($name, $attrs);
        }
    }

    /**
     * Action performed when a tag end while parsing the xml
     */
    function endTag($parser, $name){
        $this->current = $this->current->getParent();
    }

    /**
     * Action performed to store the data contained in a tag
     */
    function tagData($parser, $data){
        $this->current->addData($data);
    }

    /**
     * Returns a list of messages expressed as an array of properties
     */
    function getMessages(){
        $messages = array();
        if (($this->info != null || $this->getInfo() )&& is_array($this->info->subElements)){
            foreach ( $this->info->subElements as $message){                
                $properties = $message->getProperties();
                $properties['type'] = strtolower($message->tag);
                foreach ($message->subElements as $child){
                    if ($child->tag == 'notification'){
                        $properties = array_merge($properties, $child->getProperties());
                    }
                }
                $messages[]= $properties;
            }
        }
        return $messages;               
    }

    /**
     * Reformats a simple xml to an array
     */
    function getOneLevelArray(){
        $information = array();
        if (($this->info != null || $this->getInfo() )&& is_array($this->info->subElements))
            foreach ( $this->info->subElements as $element)
                $information[$element->tag] = $element->vardata;        
        return $information;
    }
    
    /**
    * Returns the main tag in xml string
    */
    function getMainTag(){
        if ($this->info != null || $this->getInfo() ){ return $this->info->tag;}
        return '';
    }
}


/**
 * Class that stores a tag.
 * ReadXMLMessages parses the xml information storing it as a tree of TagElements
 */
class TagElement{
    var $tag;           // Tag name                             
    var $vardata;       // Variable that contains simple data   
    var $attrs;         // Attributes                           
    var $subElements;   // Array of child elements              
    var $parent;        // Contains the parent tag

    function TagElement($tag, $parent, $attrs = array()){
        $this->tag      = $tag;
        $this->parent   = $parent;
        $this->attrs    = array();
        foreach ($attrs as $key => $value)
            $this->attrs[strtolower($key)] = $value;
        $subElements    = array();
        $this->vardata  = null;
    }

    /**
     * Adds text data to this element in the correct format attending to type parameter
     */
    function addData($data){
        if (array_key_exists('type', $this->attrs)){
                switch ($this->attrs['type']){
                case 'boolean':
                    $this->vardata = (strcmp("true", trim($data))==0);
                    break;
                case 'datetime':
                case 'date':
                    $this->vardata = strtotime($data);
                    break;
                case 'integer':
                    $this->vardata = intval($data);
                    break;
                case 'float':
                    $this->vardata = floatval($data);
                    break;
                default:
                    $this->vardata = trim($data);
            }
        }else if (!empty($data)){
            if ($this->vardata == null)
                $this->vardata = trim($data);
            else
                $this->vardata .= trim($data);
        }
    }

    /** 
     * Adds a new subElement to this element and returns the new object
     */
    function addChildElement($name, $attrs = array()){
        $child = new TagElement($name, $this, $attrs);
        $this->subElements[]= $child;
        return $child;
    }

    /**
     * Returns the parent of current object
     */
    function getParent(){
        return $this->parent;
    }

    /** 
     * Gets a simple array with the data contained in the children objects
     */
    function getproperties(){
        $array = array();
        foreach ($this->subElements as $child){
            if ($child->vardata!=null){
                $array[''.$child->tag] = $child->vardata;
            }
        }
        return $array;
    }

    function asXML(){    
        $attrib = '';
        foreach($this->attrs as $key => $value){
            $attrib .= " $key=\"$value\"";
        }        
        $content = isset($this->vardata) ? utf8_encode($this->vardata)."" : "";
        if ($this->subElements){
            foreach($this->subElements as $element){
                $content .= $element->asXML();
            }
        }
        if (strlen(trim($content)) == 0){
            $ret = "<".$this->tag."$attrib/>";
        }else{
            $ret = "<".$this->tag."$attrib>".$content."</".$this->tag.">";
        }

        if($this->parent == null)
            $ret = utf8_encode("<?xml  version=\"1.0\" encoding=\"UTF-8\"?>\n$ret\n");
        return $ret;
    }
}

?>
